'From Squeak3.7 of ''4 September 2004'' [latest update: #5989] on 22 April 2005 at 2:50:40 am'!
"Change Set:		OSProcessV3-2-5-dtl
Date:			22 April 2005
Author:			David T. Lewis

Apply 3.2.3 full, then 3.2.4, then 3.2.5.

Added OSProcess class>>allTestResults

Added UnixProcess class>>saveImageInBackground

Various changes and additions to OSProcessAccessor and UnixOSProcessAccessor to harvest some improvements by Julian, and bring released version of OSP into sync with what Avi is using for Seaside applications.

"!


!OSProcess class methodsFor: 'version testing' stamp: 'dtl 4/22/2005 02:50'!
versionString

	"OSProcess versionString"

	^'3.2.5'! !

!OSProcess class methodsFor: 'utility' stamp: 'dtl 4/11/2005 00:35'!
allTestResults
	"Run all available sUnit tests and save the results in a file named
	'OSProcessTestResults-<platform>.txt'. Display results on the Transcript as
	tests are run. Answer a string with the test results.
	Note: On some platforms, tests may produce notifier dialogs that require
	an interactive response."

	"OSProcess allTestResults"

	| resultsFileName writeStream resultString results fs problems result |
	resultsFileName _ 'OSProcessTestResults-', OSProcess platformName, '.txt'.
	[fs _ FileStream newFileNamed: resultsFileName.
	problems _ false.
	writeStream _ WriteStream on: ''.
	writeStream nextPutAll: Utilities authorInitials, ' running OSProcess unit tests ', TimeStamp now asString; cr.
	writeStream nextPutAll: 'OSProcess platformName => ' , OSProcess platformName; cr.
	writeStream nextPutAll: 'OSProcess versionInformation asString => ' , OSProcess versionInformation asString; cr.
	Transcript show: writeStream contents.
	(Class allSubInstances select: [:class | #'Tests-OSProcess' == class category])
		, (Class allSubInstances select: [:class | #'Tests-CommandShell' == class category])
		do: [:testCase | 
			writeStream nextPutAll: 'running tests in ' , testCase name; cr.
			Transcript show: 'running tests in ' , testCase name; cr.
			testCase allTestSelectors
				do: [:testSelector | 
					resultString _ (testSelector ->
						([result _ (testCase selector: testSelector) run.
						result hasPassed ifFalse: [problems _ true].
						result]
							on: Error
							do: [:ex | problems _ true.
								'caught exception ' , ex printString])) printString.
					writeStream nextPutAll: resultString; cr.
					Transcript show: resultString; cr]].
	problems 
		ifTrue: 
			[writeStream nextPutAll: 'one or more problems found'; cr.
			Transcript show: 'one or more problems were found, see results file'; cr]
		ifFalse:
			[writeStream nextPutAll: 'all tests completed without problems'; cr.
			Transcript show: 'all tests completed without problems'; cr].
	results _ writeStream contents copyReplaceAll: String cr with: String lf.
	fs nextPutAll: results]
		ensure: [fs close].
	Transcript show: 'tests results have been saved in a file named ''', resultsFileName, ''''; cr.
	^ results! !


!OSProcessAccessor methodsFor: 'file lock registry' stamp: 'dtl 4/9/2005 22:44'!
canAcquireLock: anOSFileLockDescriptor
	"Answer true if the file lock cache will permit fileLock to be acquired. This method
	does not guarantee that the underlying OS will grant the lock."

	^ (self fileLockRegistry anySatisfy:
			[:ld | ld isActive and: [ld conflictsWith: anOSFileLockDescriptor]]) not! !

!OSProcessAccessor methodsFor: 'file lock registry' stamp: 'dtl 4/10/2005 15:05'!
registeredLocksForFile: aFileStream
	"Answer all lock descriptors associated with aFileStream"

	^ self fileLockRegistry select: [:ea | ea fileStream = aFileStream]
! !

!OSProcessAccessor methodsFor: 'file lock registry' stamp: 'jf 2/26/2004 18:06'!
removeInactiveLocks
	"Go through the lock cache and remove any that have been left
	behind after their streams were closed."

	^ self fileLockRegistry copy do: [:ea | ea isActive ifFalse: [self fileLockCache remove: ea]]! !


!UnixOSProcessAccessor methodsFor: 'file locking' stamp: 'dtl 4/10/2005 15:17'!
unlockAllForFile: aFileStream
	"Unlock and uncache all locks associated with aFileStream.  This could be called
	before closing a stream, for example."

	(self registeredLocksForFile: aFileStream) do: [:ea | self unlockFileRegion: ea]
! !


!UnixProcess class methodsFor: 'utility - servers' stamp: 'dtl 4/9/2005 17:43'!
saveImageInBackground
	"When Squeak is used as a server it is sometimes desirable to periodically
	save image snapshots. This method forks a headless Squeak to perform a
	snapshot without impacting the server Squeak. Very little additional memory
	is required to do this because Unix copy-on-write memory management allows
	the two Squeak images share object memory while the save is performed.
	The saved image is given a time stamped name, and the image name of the
	main server Squeak remains unchanged."

	"UnixProcess saveImageInBackground"

	^ self forkHeadlessSqueakAndDo:
		[| newName timeStamp st | 
		timeStamp _ (TimeStamp current printString first: 19) copyReplaceAll: ':' with: '-'.
		newName _ 'squeak-' , timeStamp.
		st _ SmalltalkImage current.
		(Delay forMilliseconds: 200) wait.
		(SourceFiles at: 2) ifNotNil:
			["ensure that copying the changes file will always work"
			st closeSourceFiles.
			st saveChangesInFileNamed: (st fullNameForChangesNamed: newName)].
		st changeImageNameTo: newName,'.image';
			closeSourceFiles;
			openSourceFiles;  "so SNAPSHOT appears in new changes file"
			saveImageSegments;
			snapshot: true andQuit: true]
! !


!UnixProcess class reorganize!
('child process creation' forkHeadlessSqueakAndDo: forkHeadlessSqueakAndDoThenQuit: forkJob:arguments:environment:descriptors: forkSqueak forkSqueakAndDo: forkSqueakAndDoThenQuit:)
('external command processing' waitForCommandOutput: waitForCommandOutputArray:)
('utility' allMyChildren arguments env path programName restartVirtualMachine sessionID startSwiki:onPort:loggingTo: stdErr stdIn stdOut)
('utility - VM building' makeVmIn: makeVmIn:interactive:)
('utility - servers' saveImageInBackground)
('examples' catAFile catFromFileToFiles clientServerDemo clientServerDemo2 cloneSqueak eightLeafSqueakTree headlessChild listDirectory spawnTenHeadlessChildren squeakSqueak testEnvSet testPipe testPipeLine testRunCommand)
('platform identification' isResponsibleForThisPlatform)
('unit tests' runTests)
('X display - deprecated' canConnectToXDisplay: currentXDisplayName decapitate displayOnXServer: recapitate setXDisplayName:)
!

OSProcessAccessor removeSelector: #registryContainsRegionOverlappingWith:!

!OSProcessAccessor reorganize!
('initialize - release' initialize)
('accessing' canObtainSessionIdentifierFromPlugin sessionIdentifier)
('session identification' getSessionIdentifier getSessionIdentifierFromFileInSqueakDirectory getSessionIdentifierFromSourcesFile primGetSession)
('platform identification' isResponsibleForThisPlatform)
('testing' canAccessSystem canForwardExternalSignals)
('standard IO handles' getStdErrHandle getStdInHandle getStdOutHandle)
('pipe open' makePipeHandles)
('file control' setNonBlocking:)
('file lock registry' canAcquireLock: emulateWin32FileLocking fileLockRegistry register: registeredLocksForFile: removeInactiveLocks unregister:)
('private - IOHandle' handleFromAccessor: handleFromFileStream: ioAccessorFromSQFile:)
!

